import xml.etree.ElementTree as ET
import pandas as pd
import numpy as np
import itertools

from opta_read._auxiliares.f1_aux_funct import *

class F1:

    def __init__(self, path):
        self.path=path

    
    def results_and_td(self, team_difference=False):

        path=self.path

        file = ET.parse(path)

        # Initialize data containers
        teams = {}
        data=[]

        # Iterate through XML tree
        for season in file.getroot():
            season_name = season.attrib.get("season_name")

            for team in season.findall("Team"):
                key=team.attrib.get("uID")
                name=team.find("Name").text
                teams[key]=name

            for match in season.findall("MatchData"):
                for stats in match.findall("MatchInfo"):
                    match_day=stats.attrib.get("MatchDay")

                for stats in match.findall("TeamData"):
                    if stats.attrib.get("Side")=="Home":
                        home_team=stats.attrib.get("TeamRef")
                        score_home=stats.attrib.get("Score")
                        
                    else:
                        away_team=stats.attrib.get("TeamRef")
                        score_away=stats.attrib.get("Score")

                if team_difference==True:
                    if int(match_day)<=19:
                        home_difference=int(score_home) - int(score_away) 
                        data.append([teams[home_team], teams[away_team],int(match_day), home_difference])
                    else:
                        away_difference=int(score_away) - int(score_home)
                        data.append([teams[away_team], teams[home_team],int(match_day), away_difference])

                else:
                    result="1" if score_home > score_away else "2" if  score_home < score_away  else "x" 
                    data.append([season_name,match_day, teams[home_team],score_home,score_away, teams[away_team],result])

        if team_difference==True:
            df=pd.DataFrame(data, columns=["Local","Away","MatchDay","Match Goals Difference"])
            df = df.sort_values(by='MatchDay')

            # Agrupamos por pares de equipos y calculamos la suma acumulada
            df['Teams Goal Difference'] = (
                df
                .groupby(['Local', 'Away'])['Match Goals Difference']
                .cumsum()
            )
            df=df.reset_index(drop=True)
        else:
            df=pd.DataFrame(data, columns=["Season","MatchDay","Local","Local Goals","Away Goals","Away","Result"])

        return df
    

    def league_table(self):
        
        h2h_duels_df=self.results_and_td(team_difference=True)
        global_balance_df=global_balance(self.path)

        result = pd.DataFrame()

        for match_day in np.sort(global_balance_df["MatchDay"].unique()):
            df_filtered = global_balance_df[global_balance_df["MatchDay"] == match_day].copy()
            df_filtered['Points_adjust'] = df_filtered['Points'].astype(float)  

            # Encontramos los registros con puntos duplicados
            df_filtered_dup = df_filtered[df_filtered.duplicated(subset="Points", keep=False)]

            for puntos in df_filtered_dup["Points"].unique():
                df_filtered_pt = df_filtered_dup[df_filtered_dup["Points"] == puntos]

                # Lista de tuplas con las distintas combinaciones posibles
                teams = list(itertools.combinations(df_filtered_pt["Team"].unique(), 2))
                
                # Almacenamos el resultado de desempate para evitar llamarlo varias veces
                desempate_list = tiebreaker(h2h_duels_df, global_balance_df, match_day,teams)
                # desempate(jornada, teams)
                
                if desempate_list:
                    desempate_dict={}
                    # Map adjustments only to relevant rows
                    for x in desempate_list:
                        desempate_dict[x]=(len(desempate_list)-desempate_list.index(x))/(len(desempate_list)+1)
                    
                    for equipo, ajuste in desempate_dict.items():
                        df_filtered.loc[df_filtered["Team"] == equipo, "Points_adjust"] += ajuste

            result = pd.concat([result, df_filtered], ignore_index=True)

        # # Final cleanup
        result = result.drop_duplicates().reset_index(drop=True)

        result_sorted = result.sort_values(by=["MatchDay",'Points_adjust'], ascending=[True, False])

        # # Añadimos la columna de clasificación
        result['League Position'] = result_sorted.groupby('MatchDay').cumcount() + 1
        # # df1['Clasificación']=df1['Clasificación'].astype("string")
        result=result.sort_values(by=["MatchDay","League Position"])
        result=result.reset_index(drop=True)
        result=result.drop(columns=["Points_adjust"])  

        def plot_league_table():
            return plot_team_points(result)
        
        result.plot_league_table=plot_league_table
        
        return result  



from opta_read._auxiliares.f71_aux_funct import defensive_stats_funct

class F71:

    def __init__(self, path):
        self.path=path
    
    def defensive_stats(self):
        return defensive_stats_funct(path=self.path)

    

    

    


import xml.etree.ElementTree as ET
import pandas as pd
import plotly.graph_objects as go
from sklearn.preprocessing import MinMaxScaler
import os
import IPython as ip
    

def compare_players(df, players, stats, color_player_1, color_player_2):

    '''
    Function that returns a spider-plot comparing two players in the stats we select.
    We decide to normalize dataframe columns so that the comparison is clear

    Input:
    * df: dataframe with all players stats
    * players: list of players (max length 2) to compare
    * stats: list of stats to compare

    '''

    ## Normalize data
    df=df[stats]
    scaler = MinMaxScaler()
    df_normalized = pd.DataFrame(scaler.fit_transform(df), columns=df.columns, index=df.index)


    df1=df_normalized.loc[players[0],stats]
    df2=df_normalized.loc[players[1],stats]

    categories=stats
    fig = go.Figure()

    fig.add_trace(go.Scatterpolar(
            r=df1.tolist(),
            theta=categories,
            fill='toself',
            marker_color=color_player_1,
            marker_line_color=color_player_1,
            opacity=0.5,
            name=players[0]
    ))
    fig.add_trace(go.Scatterpolar(
            r=df2.tolist(),
            theta=categories,
            marker_color=color_player_2,
            marker_line_color=color_player_2,
            fill='toself',
            opacity=0.5,
            name=players[1]
    ))

    fig.update_layout(
        polar=dict(
        radialaxis=dict(
            visible=True
        ))
    )
    if is_jupyter()==False and is_google_colab()==False:
        fig.write_html("plots/my_plot.html")
        import webbrowser

        file_path = os.path.abspath("plots/my_plot.html")
        webbrowser.open(f"file://{file_path}")
    else:
        fig.show()


def is_jupyter():
    try:
        shell = ip.get_ipython().__class__.__name__
        return shell == 'ZMQInteractiveShell'  # Jupyter Notebook or JupyterLab
    except NameError:
        return False  # Standard Python interpreter

def is_google_colab():
    try:
        import google.colab
        return True
    except ImportError:
        return False




import xml.etree.ElementTree as ET
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib.lines import Line2D

from matplotlib import colormaps as cm
from opta_read._auxiliares.opta_pitch import *



def defensive_stats_funct(path):
    '''
    Function that returns defensive stats of a match
    '''
    file=ET.parse(path)
    # Initialize data containers
    data = []

    # Iterate through XML tree
    for game in file.getroot():
        home_team = game.attrib.get("home_team_name")
        away_team = game.attrib.get("away_team_name")

        for team in game:
            team_name = home_team if team.attrib.get("Side") == "Home" else away_team

            for player in team:
                player_name = player.attrib.get("player_name")

                for stat in player:
                    stat_name = stat.tag
                    coords = [[point.attrib.get("x"), point.attrib.get("y")] for point in stat]

                    num_events = len(coords) if stat_name != "DefensiveCoverage" else None

                    data.append([team_name, player_name, stat_name, num_events, coords])

    # Create DataFrame
    df = pd.DataFrame(data, columns=[
        "Team", "Player", "Defensive Stat", "Number of Defensive Actions", "Coords of Defensive Actions"
    ])

    def plot_defensive_coverages(team, player=None):
        return defensive_coverages(df, team, player)
    
    def plot_defensive_actions(team, player=None):
        return defensive_actions(df, team, player)

    df.plot_defensive_coverages=plot_defensive_coverages
    df.plot_defensive_actions=plot_defensive_actions
    return df

def defensive_coverages(df,team, players=None):
        '''
        Function that represents defensive coverage areas for single team or for a list of players

        '''

        # Filter dataframe by team selected and defensive coverage actions
        filtered_df=df[(df["Team"]==team) & (df["Defensive Stat"]=="DefensiveCoverage")]
        filtered_df=filtered_df[["Player","Coords of Defensive Actions"]]
        if players != None: 
            filtered_df=filtered_df[filtered_df["Player"].isin(players)]
        filtered_df.reset_index(inplace=True, drop=True)


        # Get rival team
        rival=df[df["Team"]!=team]["Team"].head(1)
        rival=rival.reset_index(drop=True)
        rival_name=rival[0]

        # Generate a color map based on number of records of filtered df

        colors = cm['tab10'].resampled(len(filtered_df))

        fig, ax = plt.subplots()

        # Add opta pitch
        opta_pitch(ax)

        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)

        # Plot defensive coverage polygons
        for i in range(len(filtered_df)):
            rect1 = patches.Polygon(filtered_df.iloc[i,1], label=filtered_df.iloc[i,0], linewidth=1,edgecolor=colors(i), facecolor=colors(i), alpha=0.5)

            ax.add_patch(rect1)

        if players==None:
            plt.title(f"Defensive coverage of {team}'s players against {rival_name}")
        else:
            plt.title(f"Defensive coverage of {' & '.join(players)} against {rival_name}")

        ax.legend()
        plt.show()

def defensive_actions(df, team, player=None):
        '''
        Function that represents defensive actions for a single team or a list of players

        '''
        
        # Filter dataframe by team selected and defensive coverage actions
        filtered_df=df[(df["Team"]==team) & (df["Defensive Stat"]!="DefensiveCoverage")]
        filtered_df=filtered_df[["Player","Defensive Stat","Coords of Defensive Actions"]]
        if player != None: 
            filtered_df=filtered_df[filtered_df["Player"].isin(player)]
        filtered_df.reset_index(inplace=True, drop=True)

        # Get rival team
        rival=df[df["Team"]!=team]["Team"].head(1)
        rival=rival.reset_index(drop=True)
        rival_name=rival[0]


        # Get list of different stats
        stats_unique=filtered_df["Defensive Stat"].unique().tolist()

        # List of possible matplotlib markers. We select only the amount we need
        markers_list=list(Line2D.markers.keys())
        markers_list.remove(",") # Remove pixel representation, as it's very small
        markers_list=markers_list[:len(stats_unique)]

        # Create dictionary that pairs a stat and a marker
        stat_marker=dict(zip(stats_unique, markers_list))

        # Generate a color map based on number of distinct players
        players_unique= filtered_df["Player"].unique().tolist()
        colors = cm['tab10'].resampled(len(players_unique))

        fig, ax = plt.subplots()

        # Add opta pitch
        opta_pitch(ax)

        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)

        player_legend=[]
        # Plot defensive coverage polygons
        for i in range(len(players_unique)):
            player_df=filtered_df[filtered_df["Player"]==players_unique[i]]
            # Create custom legend for players
            player_legend.append(Line2D([0], [0], color=colors(i), label=players_unique[i]))

            for k in range(len(player_df)):
                x,y=zip(*player_df.iloc[k,2])
                plt.scatter([float(x_coord) for x_coord in list(x)],
                        [float(y_coord) for y_coord in list(y)],
                        color=colors(i),
                        marker=stat_marker[player_df.iloc[k,1]])
                

        if player==None:    
            plt.title(f"Defensive stats of {team}'s players against {rival_name}")
        else:
            plt.title(f"Defensive stats of {' & '.join(player)} against {rival_name}")

        ax.set( xlim=[-30, 130], ylim=[-30, 130],xlabel='Possesion')

        # Create custom legend for markers
        marker_legend=[]
        for key,value in stat_marker.items():
            marker_legend.append(Line2D([0], [0], marker=value, color='w', label=key, markerfacecolor='gray', markersize=10))

        # Add first legend (markers)
        first_legend = plt.legend(handles=player_legend, title='Players', loc='upper center',ncols=5,fontsize="small")

        # # Add second legend (colors)
        second_legend = plt.legend(handles=marker_legend, title='Defensive Action', loc='lower center', ncols=5)

        # Add the first legend back manually
        plt.gca().add_artist(first_legend)
        plt.show()
    


import xml.etree.ElementTree as ET
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np
from typing import Literal

from opta_read._auxiliares.f28_aux_funct import *

class F28:

    def __init__(self, path):
        self.path=path
    
    def get_possesion(self,possesion_type:Literal["BallPossession","Territorial","TerritorialThird"], interval_length):
        if possesion_type not in ["BallPossession","Territorial","TerritorialThird"]:
            raise ValueError(f"possesion_type must be equal to one of BallPossession, Territorial or TerritorialThird")

        if interval_length not in [5,15,45]:
            raise ValueError(f"interval_length must be equal to one of 5, 15 or 45")

        path=self.path
        file=ET.parse(path)
        interval_poss = {}

        root = file.getroot()

        away_team = root.attrib.get("away_team_name")
        home_team = root.attrib.get("home_team_name")

        for possession_wave in root.findall(f".//PossessionWave[@Type='{possesion_type}']"):
            for interval_length in possession_wave.findall(f".//IntervalLength[@Type='{interval_length}']"):
                for interval in interval_length.findall("Interval"):
                    # Create a new team_poss dictionary for each interval
                    if possesion_type != "TerritorialThird":
                        team_poss = {
                            away_team: float(interval.find('Away').text),
                            home_team: float(interval.find('Home').text)
                        }
                    else:
                        team_poss = {
                            away_team: float(interval.find('Away').text),
                            home_team: float(interval.find('Home').text),
                            'middle': float(interval.find('Middle').text)
                        }
                    interval_type = interval.attrib.get("Type")
                    interval_poss[interval_type] = team_poss

        poss=pd.DataFrame.from_dict(data=interval_poss)

        def plot_pitch_possesion_evolution(color_team1="green", color_team2="blue", animated=True):
            return pitch_possesion_evolution(poss, color_team1, color_team2, animated)
        
        def plot_line_possesion_evolution():
            return line_possesion_evolution(poss)

        poss.plot_pitch_possesion_evolution=plot_pitch_possesion_evolution
        poss.plot_line_possesion_evolution=plot_line_possesion_evolution
        return poss
        # return possesion(path=self.path,possesion_type=possesion_type, interval_length=interval_length)

    
        
    




import xml.etree.ElementTree as ET
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np
from typing import Literal
from IPython.display import HTML
import IPython as ip

from opta_read._auxiliares.opta_pitch import *
import matplotlib.animation as animation



def pitch_possesion_evolution(df, color_team1="green", color_team2="blue", animated=True):
    '''
    Function that represents possesion evolution in a Opta Pitch.
    If animated=True, it will show how possesion develops over the time intervals of the match
    If animated=False, it will show mean possesion of the match
    '''

    fig, ax = plt.subplots()
    t = df.columns.tolist()

    team_0=df.index.tolist()[0]
    team_1=df.index.tolist()[1]

    l1=df.loc[team_0,t]
    l2= df.loc[team_1,t]

    if "middle" in df.index.tolist():
        l3=df.loc["middle",t]

    opta_pitch(ax)

    ax.get_yaxis().set_visible(False)

    
    if is_jupyter() or is_google_colab():
        animated_mod=False 
    else:
        animated_mod=animated

    if animated_mod==False:

        rect1 = patches.Rectangle((0, 0), np.mean(l1), 100, linewidth=1, edgecolor=color_team1, facecolor=color_team1, alpha=0.3)
        rect2 = patches.Rectangle((np.mean(l1), 0), np.mean(l2), 100, linewidth=1, edgecolor=color_team2, facecolor=color_team2, alpha=0.3)

        ax.add_patch(rect1)
        ax.add_patch(rect2)

        text1=ax.text(10,80, f"{team_0}\nmean poss:\n {round(np.mean(l1),2)}%", color=color_team1)
        text2=ax.text(80,80, f"{team_1}\nmean poss:\n {round(np.mean(l2),2)}%", color=color_team2)

        if "middle" in df.index.tolist():
            text3=ax.text(52,80, f"Middle\nmean poss:\n {round(np.mean(l3),2)}%")

        plt.title("Mean possesion of the match")

        ax.set( xlim=[0, 100], ylim=[0, 100],xlabel='Possesion')
        ax.legend()
    
    else:
        rect1 = patches.Rectangle((0, 0), l1[0], 100, linewidth=1, edgecolor=color_team1, facecolor=color_team1, alpha=0.3)
        rect2 = patches.Rectangle((l1[0], 0), l2[0], 100, linewidth=1, edgecolor=color_team2, facecolor=color_team2, alpha=0.3)
        ax.add_patch(rect1)
        ax.add_patch(rect2)

        text1=ax.text(10,80, f"{team_0} poss:\n {l1[0]}%",color=color_team1)
        text2=ax.text(80,80, f"{team_1} poss:\n {l1[0]}%", color=color_team2)

        if "middle" in df.index.tolist():
            text3=ax.text(52,80, f"Middle poss:\n {l3[0]}%")

        plt.title(f"Possesion in interval time {t[0]}")

        ax.set( xlim=[0, 100], ylim=[0, 100],xlabel='Possesion')
        ax.legend()

        def update(frame):
            # for each frame, update the data stored on each artist.
            # update the line1 plot:
            new_width1 = l1[frame]  # animate between 1 and 3
            rect1.set_width(new_width1)
            text1.set_text(f"{team_0} poss:\n {l1[frame]}%")

            new_width2 = l2[frame]  # animate between 1 and 3
            rect2.set_x(l1[frame])
            rect2.set_width(new_width2)
            text2.set_text(f"{team_1} poss:\n {l2[frame]}%")

            if "middle" in df.index.tolist():
                text3.set_text(f"Middle poss:\n {l3[frame]}%")

            plt.title(f"Possesion in interval time {t[frame]} minutes")
            # update the line2 plot:
            return (rect1, rect2)


        ani = animation.FuncAnimation(fig=fig, func=update, frames=len(t), interval=2000, repeat_delay=4000)
        # HTML(ani.to_jshtml())
    plt.show()

def line_possesion_evolution(df, color_team1="orange",color_team2="blue"):
    '''
    Function that represents possesion evolution in a line plot
    '''
    fig, ax = plt.subplots()
    t = df.columns.tolist()

    team_0=df.index.tolist()[0]
    team_1=df.index.tolist()[1]

    l1=df.loc[team_0,t]
    l2= df.loc[team_1,t]
    if "middle" in df.index.tolist():
        l3=df.loc["middle",t]
        
    ax.plot(t, l1, label=team_0, color=color_team1)[0]
    ax.plot(t, l2, label=team_1, color=color_team2)[0]
    if "middle" in df.index.tolist():
        ax.plot(t, l3, label="middle")[0]
    ax.set( xlim=[0, len(t)], ylim=[0, 100],xlabel='Interval time (mins)', ylabel='Possesion (%)')
    ax.legend()
    
    
    plt.show()

def is_jupyter():
    try:
        shell = ip.get_ipython().__class__.__name__
        return shell == 'ZMQInteractiveShell'  # Jupyter Notebook or JupyterLab
    except NameError:
        return False  # Standard Python interpreter

def is_google_colab():
    try:
        import google.colab
        return True
    except ImportError:
        return False


import matplotlib.pyplot as plt
from matplotlib.patches import Arc
from opta_read._auxiliares.opta_pitch import *

def pass_meanpos(mean_position_df, pass_matrix_df, pass_color="red",show_passes=True):

    if 'Number of Succesful Passes' not in list(pass_matrix_df.columns):
    # Raises an error if the input is a pivot table (not contains Number of Succesful Passes column)    
        raise ValueError("Input appears to be a pivot table. Input needs to be a regular dataframe.")
    
    fig, ax = plt.subplots(figsize=(18,14))
    
    max_num_pases=pass_matrix_df['Number of Succesful Passes'].max()
    # Flechas con los pases entre los jugadores con Plotter(passer, receiver, number)
    if show_passes==True:
        for i in range(len(pass_matrix_df)):
            Plotter(mean_position_df, pass_matrix_df.iloc[i,0], pass_matrix_df.iloc[i,1], pass_matrix_df.iloc[i,2], max_num_pases, pass_color)

    # Posición Media Scatter Puntos con color en función de su demarcación
    for i in range(len(mean_position_df)):
        ax.text(mean_position_df.iloc[i,1], mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,0], rotation = 45, size = 10)
        if mean_position_df.iloc[i,3] == "Goalkeeper":
            plt.scatter(x=mean_position_df.iloc[i,1], y = mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,4]*40, zorder = 1, color = "blue")
        if mean_position_df.iloc[i,3] == "Forward":
            plt.scatter(x=mean_position_df.iloc[i,1], y = mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,4]*40, zorder = 1, color = "green")
        if mean_position_df.iloc[i,3] == "Midfielder":
            plt.scatter(x=mean_position_df.iloc[i,1], y = mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,4]*40, zorder = 1, color = "grey")
        if mean_position_df.iloc[i,3] == "Defender":
            plt.scatter(x=mean_position_df.iloc[i,1], y = mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,4]*40, zorder = 1, color = "orange")
        if mean_position_df.iloc[i,3] == "Substitute":
            plt.scatter(x=mean_position_df.iloc[i,1], y = mean_position_df.iloc[i,2], s = mean_position_df.iloc[i,4]*40, zorder = 1, color = "yellow")


    opta_pitch(ax)
    

    # Quitar Ejes
    plt.axis("off")
    if show_passes==True:
        plt.title("Passmap usando fichero F27 Opta - STATS Perform")
    else:
        plt.title("Posición media usando fichero F27 Opta - STATS Perform")

    plt.show()


import xml.etree.ElementTree as ET
import pandas as pd
from opta_read._auxiliares.f30_aux_funct import *

class F30:

    def __init__(self, path):
        self.path=path
    
    def team_stats(self):
        '''
        Function that returns season stats of a team
        '''
        file=ET.parse(self.path)
        team_dict={}
        for team in file.getroot():
            for stat in team:
                attribute=stat.attrib.get("name")
                value=stat.text
                team_dict[attribute]=[value]
        del team_dict[None]

        team_stats=pd.DataFrame.from_dict(data=team_dict, orient="index", columns=[team.attrib.get("name")])

        return team_stats

    def players_stats(self):
        '''
        Function that returns season stats of all players of a team
        '''
        players_dict={}
        player_stats={}
        path=self.path
        file=ET.parse(path)

        root=file.getroot()
        for player in root.findall(".//Player"):
            first_name=player.attrib.get("first_name")
            last_name=player.attrib.get("last_name")
            full_name=first_name + " " + last_name
            for stat in player:
                attribute=stat.attrib.get("name")
                value=stat.text
                player_stats[attribute] = float(value) 
            
            players_dict[full_name]=player_stats
            player_stats={}
        
        players_stats=pd.DataFrame.from_dict(data=players_dict, orient="index")

        def plot_compare_players(players, stats, color_player_1, color_player_2):
            return compare_players(players_stats, players, stats, color_player_1, color_player_2)
        players_stats.plot_compare_players=plot_compare_players

        return players_stats
    



import matplotlib.pyplot as plt
from matplotlib.patches import Arc


def opta_pitch(ax):        
    # OPTA PITCH
    # Pitch Outline & Centre Line
    plt.plot([0,0],[0,100], color="black")
    plt.plot([0,100],[100,100], color="black") #upper line x_start x_end y_start y_end
    plt.plot([100,100],[100,0], color="black")
    plt.plot([100,0],[0,0], color="black")
    plt.plot([50,50],[0,100], color="black")

    # Left Penalty Area
    plt.plot([17,17],[78.9,21.1],color="black")
    plt.plot([0,17],[78.9,78.9],color="black")
    plt.plot([17,0],[21.1,21.1],color="black")

    # Right Penalty Area
    plt.plot([100,83],[78.9,78.9],color="black")
    plt.plot([83,83],[78.9,21.1],color="black")
    plt.plot([83,100],[21.1,21.1],color="black")

    # Left 6-yard Box
    plt.plot([0,5.8],[63.2,63.2],color="black")
    plt.plot([5.8,5.8],[63.2,36.8],color="black")
    plt.plot([5.8,0],[36.8,36.8],color="black")

    # Right 6-yard Box
    plt.plot([100,94.2],[63.2,63.2],color="black")
    plt.plot([94.2,94.2],[63.2,36.8],color="black")
    plt.plot([94.2,100],[36.8,36.8],color="black")

    # Prepare Circles OK
    centreCircle = plt.Circle((50,50),9.15,color="black",fill=False)
    centreSpot = plt.Circle((50,50),0.6,color="black")
    leftPenSpot = plt.Circle((11.5,50),0.6,color="black")
    rightPenSpot = plt.Circle((88.5,50),0.6,color="black")

    # Draw Circles
    ax.add_patch(centreCircle)
    ax.add_patch(centreSpot)
    ax.add_patch(leftPenSpot)
    ax.add_patch(rightPenSpot)

    # Prepare Arcs based on penalty Spots
    leftArc = Arc((11.5,50),height=18.3,width=18.3,angle=0,
                theta1=310,theta2=50,color="black")
    rightArc = Arc((88.5,50),height=18.3,width=18.3,angle=0,theta1=130,theta2=230,color="black")


    # Draw Arcs
    ax.add_patch(leftArc)
    ax.add_patch(rightArc)


def Plotter(mean_position, passer, receiver, number, num_max_pases, pass_color="red"):

        '''
        Function that plots a line between the mean position of passer and receiver
        where the width of that line is based on the number of passes that 
        take place between those two players.

        Input:
        * mean_position: dataframe with each player and its mean position
        * passer: name of the passer
        * receiver: name of the receiver 
        * number: amount of passes between passer and receiver
        * num_max_pases: maximum amount of passes between two players considering the whole team to calculate it.
        * pass_color: color used to represent passes in the plot
        
        '''

        mp_coords=mean_position[['Passer','x','y']].drop_duplicates()
        mp_coords=mp_coords.set_index("Passer")

        passer_loc = [[float(mp_coords.loc[passer].iloc[0]), float(mp_coords.loc[passer].iloc[1])]]
        receiver_loc = [[float(mp_coords.loc[receiver].iloc[0]), float(mp_coords.loc[receiver].iloc[1])]]

        for i,j in zip(passer_loc, receiver_loc):
            plt.arrow(x = i[0],
                    y = i[1],
                    dx = (j[0]-i[0]),
                    dy = (j[1]-i[1]),
                    linewidth = int(number),
                    alpha = (int(number)/num_max_pases), # propiedad transparencia que va de 0 a 1
                    length_includes_head=True,
                    color = pass_color)


import xml.etree.ElementTree as ET
import pandas as pd

from opta_read._auxiliares.f27_aux_funct import *

class F27:

    def __init__(self, path):
        self.path=path

    def mean_position(self):

        ''''
        Function that returns a dataframe with all players found in a F27 file, together with
        its mean position based on contacts with ball and the number of sucessful passes

        Input:
        OPTA F27 xml file


        '''
        xml_file=self.path

        file = ET.parse(xml_file)

        namelist= []
        xlist =[]
        ylist = []
        poslist = []
        pass_success_list = []

        for node in file.getroot():
            name = node.attrib.get("player_name")
            x = node.attrib.get("x")
            y = node.attrib.get("y")
            pos = node.attrib.get("position")
            pass_success = node.attrib.get("pass_success")
            namelist.append(name)
            xlist.append(x)
            ylist.append(y)
            poslist.append(pos)
            pass_success_list.append(pass_success)


        
        df=pd.DataFrame(data = list(zip(namelist,xlist,ylist,poslist, pass_success_list)),
                        columns = ["Passer","x","y","Position_Passer","PassSuccess"])

        df['x'] = pd.to_numeric(df['x'], errors='coerce')
        df['y'] = pd.to_numeric(df['y'], errors='coerce')                
        df['PassSuccess'] = pd.to_numeric(df['PassSuccess'], errors='coerce').fillna(0).astype(int) 
        
        return df.sort_values(by=["Position_Passer",'PassSuccess'],ascending=[True,False]).reset_index(drop=True)
    
    def pass_matrix(self, pivot_table=False, color=False):
        '''
        Function that returns a dataframe with all players found in a F27 file, together with
        its mean position based on contacts with ball and the number of sucessful passes

        Input:
        * xml_file: OPTA F27 xml file
        * pivot_table: if true, function will return a frequency table. If not, 
        it will return a regular dataframe with columns "Passer","Receiver" and "Number of Succesful Passes"
        * color: if true, it fills number of passes cells based on its value.

        '''
        xml_file=self.path

        file = ET.parse(xml_file)

        passer = []
        receiver = []
        passeslist = []

        for node in file.getroot():
            for players in node:
                passes = players.text
                name = players.attrib.get("player_name")
                passer.append(node.attrib.get("player_name"))
                receiver.append(name)
                passeslist.append(passes)


        df=pd.DataFrame(data = list(zip(passer,receiver,passeslist)),
                        columns = ["Passer","Receiver","Number of Succesful Passes"])
        df["Number of Succesful Passes"]=df["Number of Succesful Passes"].astype(int)
        
        if pivot_table==True:
        # Crear la tabla de frecuencias
            df = df.pivot_table(values='Number of Succesful Passes', index='Passer', columns='Receiver',aggfunc="sum", fill_value=0)
        
        if color==True:
            return df.style.background_gradient( axis=None)
        else:
            return df
        
    
    def plot_pass_meanpos(self,pass_color="red",show_passes=True):
        pass_matr=self.pass_matrix()
        return pass_meanpos(self.mean_position(),pass_matr,pass_color, show_passes)
            
    
    
            



import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

VERSION = '0.0.2' 
PACKAGE_NAME = 'opta_read'  
AUTHOR = 'Felix Suarez'  
URL = 'https://github.com/felsuacor/BD_deporte' 

LICENSE = 'MIT' #Tipo de licencia
DESCRIPTION = 'Librería para leer ficheros de xml de Opta y extraer la información' 
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') #Referencia al documento README con una descripción más elaborada
LONG_DESC_TYPE = "text/markdown"


#Paquetes necesarios para que funcione la libreía. Se instalarán a la vez si no lo tuvieras ya instalado
INSTALL_REQUIRES = [
      'pandas',
      'numpy',
      'more-itertools',
      'matplotlib',
      'seaborn',
      'typing_extensions',
      'networkx',
      'scikit-learn',
      'plotly'
      ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True
)


import xml.etree.ElementTree as ET
import pandas as pd
import numpy as np
import networkx as nx  
import matplotlib.pyplot as plt
 
from matplotlib import colormaps as cm
import seaborn as sns



def global_balance(path):

    file = ET.parse(path)

    # Initialize data containers
    teams = {}
    data=[]

    # Iterate through XML tree
    for season in file.getroot():
        season_name = season.attrib.get("season_name")

        for team in season.findall("Team"):
            key=team.attrib.get("uID")
            name=team.find("Name").text
            teams[key]=name

        for match in season.findall("MatchData"):
            for stats in match.findall("MatchInfo"):
                jornada=stats.attrib.get("MatchDay")

            for stats in match.findall("TeamData"):
                if stats.attrib.get("Side")=="Home":
                    home_team=stats.attrib.get("TeamRef")
                    score_home=stats.attrib.get("Score")
                    
                else:
                    away_team=stats.attrib.get("TeamRef")
                    score_away=stats.attrib.get("Score")

            difference_home=int(score_home) - int(score_away)
            difference_away=int(score_away) - int(score_home)
            
            points_home=3 if difference_home > 0 else 0 if  difference_home < 0  else 1 
            points_away=3 if difference_away > 0 else 0 if  difference_away < 0  else 1 

            data.append([season_name,int(jornada), teams[home_team],int(score_home),difference_home,points_home])
            data.append([season_name,int(jornada), teams[away_team],int(score_away),difference_away,points_away])

    df1=pd.DataFrame(data, columns=["Season","MatchDay","Team","Goals scored","Goals Difference","Points"])
    df1['Points'] = df1.groupby('Team')['Points'].cumsum()
    df1['Goals Difference'] = df1.groupby('Team')['Goals Difference'].cumsum()
    df1['Goals scored'] = df1.groupby('Team')['Goals scored'].cumsum()
    df1=df1.reset_index(drop=True)

    return df1

def tiebreaker(h2h_duels_df, global_balance_df, match_day,teams_list):
    lista=[]
    for teams in teams_list:
        team_a, team_b = teams
        h2h_duels_filtered=h2h_duels_df[(h2h_duels_df["Local"]==team_a) & (h2h_duels_df["Away"]==team_b)]
        if h2h_duels_filtered.empty==True:
            h2h_duels_filtered=h2h_duels_df[(h2h_duels_df["Away"]==team_b) & (h2h_duels_df["Away"]==team_a)]

        h2h_duels_filtered_matchday=h2h_duels_filtered[h2h_duels_filtered["MatchDay"]<= match_day].sort_values(by="MatchDay").head(1)
        h2h_duels_filtered_matchday=h2h_duels_filtered_matchday.reset_index(drop=True)
        ### Criterio diferencia de goles en duelos directos
        if (h2h_duels_filtered_matchday.empty==False) and (h2h_duels_filtered_matchday["Teams Goal Difference"] != 0).all():
            if (h2h_duels_filtered_matchday["Teams Goal Difference"]<0).all():
                lista.append([h2h_duels_filtered_matchday.loc[0,"Away"],h2h_duels_filtered_matchday.loc[0,"Local"]])
                
            elif (h2h_duels_filtered_matchday["Teams Goal Difference"] > 0).all():
                lista.append([h2h_duels_filtered_matchday.loc[0,"Local"],h2h_duels_filtered_matchday.loc[0,"Away"]])

        ### Criterio diferencia de goles acumulada
        else: 
            balance=global_balance_df[global_balance_df["MatchDay"]==match_day]
            balance_team_a=int(balance[balance["Team"]==team_a]["Goals Difference"].values[0])      
            balance_team_b=int(balance[balance["Team"]==team_b]["Goals Difference"].values[0])
            if balance_team_a > balance_team_b:
                lista.append([team_a,team_b])
            elif balance_team_a < balance_team_b:
                lista.append([team_b,team_a])
            else: 
                ### Criterio goles a favor
                goles_team_a=int(balance[balance["Team"]==team_a]["Goals scored"].values[0])
                goles_team_b=int(balance[balance["Team"]==team_b]["Goals scored"].values[0])
                if goles_team_a > goles_team_b:
                    lista.append([team_a,team_b])
                elif goles_team_a < goles_team_b:
                    lista.append([team_b,team_a])  
        
    # print(lista)
    G = nx.DiGraph()
    G.add_edges_from(lista)
    # Función para romper ciclos
    def break_cycles(graph):
        graph_copy = graph.copy()
        while True:
            try:
                list(nx.topological_sort(graph_copy))
                break  # No hay ciclos
            except nx.NetworkXUnfeasible:
                cycle = list(nx.find_cycle(graph_copy, orientation='original'))
                edge_to_remove = cycle[-1][:2]  # Eliminar última arista del ciclo
                graph_copy.remove_edge(*edge_to_remove)
        return graph_copy

    # Aplicar algoritmo
    G_acyclic= break_cycles(G)

    # Perform topological sort
    sorted_list = list(nx.topological_sort(G_acyclic))

    return sorted_list


def plot_team_points(league_table_df):
    # Estilo visual
    sns.set_style("whitegrid")

    # Crear DataFrame
    df = league_table_df[["MatchDay","Team","League Position"]]

    # Crear figura y eje principal
    fig, ax1 = plt.subplots(figsize=(20, 10))

    # Eje Y izquierdo - Equipo
    colores_equipos = dict(zip(df['Team'].unique(), sns.color_palette("tab20", len(df['Team'].unique()))))
    
    ax1.set_xlabel('MatchDay')
    ax1.set_ylabel('Team')
    jornada_1=df[df['MatchDay']==1].sort_values("League Position",ascending=False)
    ax1.plot(jornada_1['MatchDay'], jornada_1['Team'], marker=',',color="white", label='Team')
    ax1.tick_params(axis='y')

    # Eje Y derecho - League Position
    ax2 = ax1.twinx()

    ax2.set_ylabel('League Position')
    for equipo in np.sort(df['Team'].unique()):
        datos_equipo = df[df['Team'] == equipo]
        ax2.plot(datos_equipo['MatchDay'], datos_equipo['League Position'],color=colores_equipos[equipo], linestyle='--', marker='s', label='League Position')

    min_y=int(min(jornada_1["League Position"]))
    max_y=int(max(jornada_1["League Position"]))
    ax2.set_yticks(range(min_y, max_y + 1))
    # ax2.tick_params(axis='y', labelcolor=color_clasificacion)
    ax2.set_ylim(ax2.get_ylim()[::-1])

    # Título y leyenda
    fig.suptitle("Development of team's League Position by Match Day", fontsize=14)
    fig.tight_layout()
    plt.show()
